    angular.module("Tasks", ["ui.router","ngResource"])
    .config(function($stateProvider,$urlRouterProvider) {

        $urlRouterProvider.otherwise('/list');
        $stateProvider.state('list', {
             cache: false,
             url: '/list',
             templateUrl: '/personal/sgannon/BlogPost/SiteAssets/BlogPost/templates/tasklist.html',
             controller: 'TaskController'
        })
        .state('detail', {
            url: '/:id',
             templateUrl: '/personal/sgannon/BlogPost/SiteAssets/BlogPost/templates/taskdetail.html',
             controller: 'TaskController'
        });
    })
    .service("TaskFactory", ['$http','$q', function($http,$q) {
    	var getUrl = function(id) {
            var retVal = '';
    		if (!id) {
    			//get all items
    			retVal = "../_api/web/lists/GetByTitle('ToDos')/items";
    		} else {
    			//get a single item
    			retVal = "../_api/web/lists/GetByTitle('ToDos')/items(" + id + ")";
    		}
            return retVal;
    	};
        $http({ 
            url: "/personal/sgannon/BlogPost/_api/web/lists/GetByTitle('ToDos')",
            method: "GET",
            headers: {
                accept :"application/json;odata=verbose"
            }
        }).then(function(responseData) {
            this.contentType = responseData.data.d.ListItemEntityTypeFullName;
        })
    	return {
    		query: function() {
    			var $returnValue = $q.defer();
    			$returnValue = $http({
    				url: getUrl(),
    				method: 'GET',
                    isArray: true,
                    headers: { 'Accept': 'application/json;odata=verbose'}
    			});
                return $returnValue;
    		}
    		,
    		get : function(id) {
                var $returnValue = $q.defer();
                //Get a single item
                $returnValue = $http({
                    method: "GET",
                    url: getUrl(id.id),
                    headers: {
                        accept: "application/json;odata=verbose"
                    }
                });
                return $returnValue;
    		},
            update: function(item, taskData) {
                var $returnValue = $q.defer();
                $returnValue = $.ajax({
                    url: taskData.__metadata.uri,
                    type: "POST",
                    contentType: "application/json;odata=verbose",
                    data: JSON.stringify(item),
                    headers: {
                        "Accept": "application/json;odata=verbose",
                        "X-RequestDigest": $("#__REQUESTDIGEST").val(),
                        "X-HTTP-Method": "MERGE",
                        "If-Match": taskData.__metadata.etag
                    }
                });
                return $returnValue;
            },
    		// put : function(data) {
    		//	Save new item
    		
    		// },
    		remove : function(id) {
                var $returnValue = $q.defer;
                $returnValue = $http({
                    url: getUrl(id),
                    method: 'POST',
                    headers: {
                        'Accept': 'application/json;odata=verbose',
                        'content-type': 'application/json;odata=verbose',
                        'X-HTTP-Method': 'DELETE',
                        'X-RequestDigest': $("#__REQUESTDIGEST").val(),
                        'If-Match': '*'
                    }
    		});
            return $returnValue;
        }
    }
    }]).controller('TaskController',['$scope', '$http', '$state', '$stateParams', 'TaskFactory', function($scope,$http,$state,$stateParams,TaskFactory) {
        switch ($state.current.url) {
            case "/:id":
                if ($stateParams.id){
                    //This is an existing task
                    var id = $stateParams.id;
                    TaskFactory.get({ id: id }).then(function(responseData) {
                    	var item = responseData.data.d;
                        item.DueDate = $scope.editDate(item.DueDate);
                        $scope.task = item;
                    });
                } else {
                    //This is a new task
                    $scope.task = new TaskFactory();
                };
                break;
            case "/list":
                var tasks = TaskFactory.query().then(function(responseData) {
                    $scope.tasks = [];
                	$(responseData.data.d.results).each(function(index, item) {
                		item.DueDate = $scope.editDate(item.DueDate);
                        $scope.tasks.push(item);
                	});
                }, function(error) {
                    console.log(error);
                });
                break;
            default:
                break;
        }

        $scope.saveTask = function() {
            if (!$scope.task.Id) {
				//Save new item
            } else {
                var requestBody = {};
                requestBody.__metadata = { type: $scope.task.__metadata.type };
                requestBody.Title = $scope.task.Title;
                requestBody.DueDate = $scope.transformDate($scope.task.DueDate);
                requestBody.Id = $scope.task.Id;
                TaskFactory.update(requestBody, $scope.task).then(function() {
                    alert('Task successfully saved.');
                    $state.go('list', {}, { reload: true });
                },
                    function(error) {
                        alert(error.message);
                        $state.go('list');
                });
            }
        };
        $scope.cancelTask = function() {
            $state.go('list');
        };
        $scope.deleteTask = function() {
            TaskFactory.remove($scope.task.Id).then(
                function() {
                    alert('Task successfully deleted.');
                    $state.go('list', {}, { reload: true });
                },
                function(error) {
                    alert(error.message);
                    $state.go('list', {}, { reload: true });
                });
        };
        $scope.editDate = function(inDate) {
            var retDate = new Date(inDate)
            return retDate.toDateString();
        };
        $scope.transformDate = function(inDate) {
            var retDate = new Date(inDate)
            return retDate.toUTCString();
        };

    }])